import { CommonModule } from '@angular/common';
import { Injectable, NgModule, Component, Input, EventEmitter, Output, defineInjectable } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AngularTreeViewService {
    constructor() { }
    /**
     * @param {?} data
     * @param {?} processed_data
     * @param {?} expand_tracker
     * @param {?} configs
     * @return {?}
     */
    processData(data, processed_data, expand_tracker, configs) {
        // let top = data.reduce((a, b) => a.parent < b.parent ? a : b);
        // let top = data.reduce((a, b) => a.parent < b.parent ? a : b);
        /** @type {?} */
        const top = 0;
        data.map((/**
         * @param {?} rec
         * @return {?}
         */
        rec => {
            rec.pathx = [];
            rec.leaf = false;
        }));
        /** @type {?} */
        const children = this.findChildren(data, top, configs);
        this.orderData(data, processed_data, configs, children, [], 0);
        processed_data.map((/**
         * @param {?} rec
         * @return {?}
         */
        rec => {
            /** @type {?} */
            const parent_pathx = rec.parent_pathx;
            rec.parent_pathx = parent_pathx.join('.');
            parent_pathx.push(rec[configs.id_field]);
            // Add current id to create current path.
            rec.pathx = parent_pathx.join('.');
            expand_tracker[rec.pathx] = false;
        }));
        // Expand root.
        expand_tracker[''] = true;
    }
    /**
     * @param {?} data
     * @param {?} id
     * @param {?} configs
     * @return {?}
     */
    findChildren(data, id, configs) {
        return data.filter((/**
         * @param {?} rec
         * @return {?}
         */
        rec => rec[configs.parent_id_field] === id));
    }
    /**
     * @param {?} data
     * @param {?} processed_data
     * @param {?} configs
     * @param {?} parents
     * @param {?} paths
     * @param {?} levelx
     * @return {?}
     */
    orderData(data, processed_data, configs, parents, paths, levelx) {
        parents.forEach((/**
         * @param {?} parent
         * @return {?}
         */
        parent => {
            /** @type {?} */
            const children = this.findChildren(data, parent[configs.id_field], configs);
            if (children.length === 0) {
                parent.leaf = true;
                parent.levelx = levelx;
                parent.parent_pathx = [...paths];
                processed_data.push(parent);
            }
            else {
                parent.parent_pathx = [...paths];
                parent.levelx = levelx;
                processed_data.push(parent);
                /** @type {?} */
                const newPaths = [...paths, parent[configs.id_field]];
                this.orderData(data, processed_data, configs, children, newPaths, levelx + 1);
            }
        }));
    }
}
AngularTreeViewService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
AngularTreeViewService.ctorParameters = () => [];
/** @nocollapse */ AngularTreeViewService.ngInjectableDef = defineInjectable({ factory: function AngularTreeViewService_Factory() { return new AngularTreeViewService(); }, token: AngularTreeViewService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AngularTreeViewComponent {
    /**
     * @param {?} angularTreeViewService
     */
    constructor(angularTreeViewService) {
        this.angularTreeViewService = angularTreeViewService;
        this.processed_data = [];
        this.expand_tracker = {};
        this.columns = [];
        this.default_configs = {
            css: {
                expand_class: 'plus',
                collapse_class: 'minus',
                add_class: 'plus',
                edit_class: '',
                delete_class: '',
                save_class: '',
                cancel_class: ''
            },
            data_loading_text: 'Loading...',
            row_class_function: (/**
             * @return {?}
             */
            () => true)
        };
        this.default_column_config = {
            sorted: 0,
            sort_type: null,
            editable: false,
            hidden: false,
            sortable: true
        };
        this.cellclick = new EventEmitter();
        this.expand = new EventEmitter();
        this.collapse = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        this.setDefaultConfigs();
        this.setColumnNames();
        this.angularTreeViewService.processData(this.data, this.processed_data, this.expand_tracker, this.configs);
    }
    /**
     * @return {?}
     */
    setDefaultConfigs() {
        this.processed_data = [];
        this.configs = Object.assign({}, this.default_configs, this.configs);
        if (!this.configs.id_field) {
            window.console.error('id field is mandatory!');
        }
        if (!this.configs.parent_id_field) {
            window.console.error('parent_id field is mandatory!');
        }
    }
    /**
     * @return {?}
     */
    setColumnNames() {
        this.columns = this.configs.columns ? this.configs.columns : [];
        // If columns doesn't exist in user's object.
        if (!this.configs.columns) {
            /** @type {?} */
            const column_keys = Object.keys(this.data[0]);
            // Insert Header and default configuration.
            column_keys.forEach((/**
             * @param {?} key
             * @return {?}
             */
            key => {
                this.columns.push(Object.assign({ 'header': key, 'name': key }, this.default_column_config));
            }));
        }
        else {
            // Insert Header and default configuration.
            for (let i = 0; i < this.columns.length; i++) {
                this.columns[i] = Object.assign({}, this.default_column_config, this.columns[i]);
            }
        }
    }
}
AngularTreeViewComponent.decorators = [
    { type: Component, args: [{
                selector: 'db-angular-tree-view',
                template: "<table class=\"db-tree-view\">\r\n    <thead \r\n        db-tree-head\r\n        [processed_data]=\"processed_data\"\r\n        [expand_tracker]=\"expand_tracker\"\r\n        [data]=\"data\"\r\n        [columns]=\"columns\"\r\n        [configs]=\"configs\">\r\n    </thead>\r\n\r\n    <tbody \r\n        db-tree-body\r\n        [processed_data]=\"processed_data\"\r\n        [expand_tracker]=\"expand_tracker\"\r\n        [data]=\"data\"\r\n        [columns]=\"columns\"\r\n        [configs]=\"configs\"\r\n        [cellclick]=\"cellclick\"\r\n        [expand]=\"expand\"\r\n        [collapse]=\"collapse\"\r\n    >        \r\n    </tbody>\r\n</table>",
                styles: [".db-tree-view{line-height:1.5em;border-collapse:collapse;border-spacing:0;display:table;width:100%;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all;border:1px solid gray;color:#4b4b4b}"]
            }] }
];
/** @nocollapse */
AngularTreeViewComponent.ctorParameters = () => [
    { type: AngularTreeViewService }
];
AngularTreeViewComponent.propDecorators = {
    data: [{ type: Input }],
    configs: [{ type: Input }],
    cellclick: [{ type: Output }],
    expand: [{ type: Output }],
    collapse: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TreeBodyComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onRowExpand(event) {
        /** @type {?} */
        const row_data = event.data;
        if (this.expand_tracker[row_data.pathx]) {
            this.collapse.emit(event);
        }
        else {
            this.expand.emit(event);
        }
        this.expand_tracker[row_data.pathx] = !this.expand_tracker[row_data.pathx];
    }
}
TreeBodyComponent.decorators = [
    { type: Component, args: [{
                selector: '[db-tree-body]',
                template: "<tr *ngFor=\"let rec of processed_data\">\n  <ng-container *ngIf=\"expand_tracker[rec.parent_pathx]\">\n    <td *ngFor=\"let column of columns; index as i\">\n      <db-tree-cell\n        [configs]=\"configs\"\n        [column]=\"column\"\n        [index]=\"i\"\n        [row_data]=\"rec\"\n        [expand_tracker]=\"expand_tracker\"\n        [cellclick]=\"cellclick\"\n        (rowexpand)=\"onRowExpand($event)\"\n      >\n      </db-tree-cell>\n    </td>\n  </ng-container>\n</tr>",
                styles: [".db-tree-view{line-height:1.5em;border-collapse:collapse;border-spacing:0;display:table;width:100%;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all;border:1px solid gray;color:#4b4b4b}tr.child{background:#fff}tr.child td:nth-child(2){padding:.875rem 1.25rem .875rem 2.5rem!important}tr.parent{background:#fafbff}tr.selected{background-color:#e2e7eb}tr span.parent_container{padding-left:45px}tr td{border:1px solid #cdd5dc;vertical-align:middle;position:relative;padding:.5rem}td.clear-left-border{border-left:0!important}td.clear-right-border{border-right:0!important}.column-hide{display:none}"]
            }] }
];
/** @nocollapse */
TreeBodyComponent.ctorParameters = () => [];
TreeBodyComponent.propDecorators = {
    data: [{ type: Input }],
    configs: [{ type: Input }],
    processed_data: [{ type: Input }],
    expand_tracker: [{ type: Input }],
    columns: [{ type: Input }],
    cellclick: [{ type: Input }],
    expand: [{ type: Input }],
    collapse: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TreeCellComponent {
    constructor() {
        this.rowexpand = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.is_expand_column = this.index === 0;
        this.show_expand_icon = !this.row_data.leaf;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onCellClick(event) {
        if (this.index === 0 && !this.row_data.leaf) {
            this.rowexpand.emit({ event: event, data: this.row_data });
        }
        this.cellclick.emit({ event: event, data: this.row_data });
    }
}
TreeCellComponent.decorators = [
    { type: Component, args: [{
                selector: 'db-tree-cell',
                template: "<div (click)=\"onCellClick($event)\">\n  <ng-container *ngIf=\"is_expand_column\">\n    <div [ngStyle]=\"{'padding-left': row_data.leaf ? row_data.levelx * 20 + 'px' : row_data.levelx * 10 + 'px'}\">\n      <ng-container *ngIf=\"!expand_tracker[row_data.pathx]\">\n          <i \n          [ngClass]=\"configs.css.expand_class\" \n          class=\"expand-icon\" \n          aria-hidden=\"true\"\n          *ngIf=\"show_expand_icon\"\n        ></i>\n      </ng-container>\n      <ng-container *ngIf=\"expand_tracker[row_data.pathx]\">\n          <i \n          [ngClass]=\"configs.css.collapse_class\" \n          class=\"expand-icon\" \n          aria-hidden=\"true\"\n          *ngIf=\"show_expand_icon\"\n        ></i>\n      </ng-container>      \n      <span>{{row_data[column.name]}}</span>\n    </div>\n  </ng-container>\n  <ng-container *ngIf=\"!is_expand_column\">\n      <span>{{row_data[column.name]}}</span>\n  </ng-container>  \n</div>\n",
                styles: [".db-tree-view{line-height:1.5em;border-collapse:collapse;border-spacing:0;display:table;width:100%;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all;border:1px solid gray;color:#4b4b4b}i.expand-icon{margin-right:33px}i.plus{border:1px solid transparent}i.plus:before{content:'';width:18px;height:1px;border-top:2px solid #222;display:block;position:absolute;margin-top:-15px;margin-left:6px}i.plus:after{content:'';width:1px;height:18px;border-right:2px solid #222;display:block;position:absolute;margin-top:-23px;margin-left:14px}i.minus{border:1px solid transparent}i.minus:before{content:'';width:18px;height:1px;border-top:2px solid #222;display:block;position:absolute;margin-top:-15px;margin-left:6px}"]
            }] }
];
/** @nocollapse */
TreeCellComponent.ctorParameters = () => [];
TreeCellComponent.propDecorators = {
    configs: [{ type: Input }],
    index: [{ type: Input }],
    row_data: [{ type: Input }],
    column: [{ type: Input }],
    expand_tracker: [{ type: Input }],
    cellclick: [{ type: Input }],
    rowexpand: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TreeCellModule {
}
TreeCellModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TreeCellComponent],
                imports: [
                    CommonModule
                ],
                exports: [TreeCellComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TreeBodyModule {
}
TreeBodyModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TreeBodyComponent],
                imports: [
                    CommonModule,
                    TreeCellModule
                ],
                exports: [TreeBodyComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TreeHeadComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
}
TreeHeadComponent.decorators = [
    { type: Component, args: [{
                selector: '[db-tree-head]',
                template: "<tr>\n  <th *ngFor=\"let column of columns\">\n    {{column.header}}\n  </th>\n</tr>",
                styles: [".db-tree-view{line-height:1.5em;border-collapse:collapse;border-spacing:0;display:table;width:100%;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all;border:1px solid gray;color:#4b4b4b}tr{background:#fff}tr th{padding:.5rem;font-family:Exo;font-size:1rem;font-weight:600;line-height:1.25;color:#181818;border:1px solid #cdd5dc;vertical-align:middle;position:relative}tr th.sortable{cursor:pointer}tr th.sortable:hover span.arrow-up{border-bottom:6px solid #afafaf}tr th.sortable:hover span.arrow-down{border-top:6px solid #afafaf}tr th span.arrow-up{width:0;height:0;border-left:5px solid transparent;border-right:5px solid transparent;border-bottom:6px solid #ddd;position:relative;top:-16px}tr th span.arrow-up.active{border-bottom:6px solid #8a8989!important}tr th span.arrow-down{width:0;height:0;border-left:5px solid transparent;border-right:5px solid transparent;border-top:6px solid #ddd;position:relative;left:10px;top:16px}tr th span.arrow-down.active{border-top:6px solid #8a8989!important}th.clear-left-border{border-left:0!important}th.clear-right-border{border-right:0!important}.column-hide{display:none}svg{width:25px;padding-right:5px}"]
            }] }
];
/** @nocollapse */
TreeHeadComponent.ctorParameters = () => [];
TreeHeadComponent.propDecorators = {
    data: [{ type: Input }],
    configs: [{ type: Input }],
    processed_data: [{ type: Input }],
    expand_tracker: [{ type: Input }],
    columns: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TreeHeadModule {
}
TreeHeadModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TreeHeadComponent],
                imports: [
                    CommonModule
                ],
                exports: [TreeHeadComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AngularTreeViewModule {
}
AngularTreeViewModule.decorators = [
    { type: NgModule, args: [{
                declarations: [AngularTreeViewComponent],
                imports: [
                    CommonModule,
                    TreeBodyModule,
                    TreeHeadModule
                ],
                exports: [AngularTreeViewComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AngularTreeViewService, AngularTreeViewComponent, AngularTreeViewModule, TreeBodyComponent as ɵb, TreeBodyModule as ɵa, TreeCellComponent as ɵd, TreeCellModule as ɵc, TreeHeadComponent as ɵf, TreeHeadModule as ɵe };

//# sourceMappingURL=angular-tree-grid.js.map